# mounts rootfs rw

service system/mountroot {
	need = system/initial;

	script start = {
		# Read functions
		. /etc/rc.d/init.d/functions

		# And sysconfig
		if [ -r /etc/sysconfig/system ]; then
			. /etc/sysconfig/system
		else
			VSERVER=no
		fi

		if [ "$VSERVER" = "yes" ]; then
			exit 0
		fi

		if [ -f /fastboot ] || grep -iq "fastboot" /proc/cmdline 2> /dev/null ; then
			fastboot=yes
		else
			fastboot=
		fi

		if [ -f /fsckoptions ]; then
			fsckoptions=$(cat /fsckoptions)
		else
			fsckoptions=''
		fi

		if [ -f /forcefsck ]; then
				fsckoptions="-f $fsckoptions"
		fi

		_RUN_QUOTACHECK=0
		_ROOTFS_TYPE=$(grep " / " /proc/mounts | awk '{ print $3 }')

		if [ -z "$fastboot" -a "$_ROOTFS_TYPE" != "nfs" -a "$_ROOTFS_TYPE" != "romfs" ]; then 
			show "Checking root filesystem"; started
			initlog -c "fsck -C -T -a $fsckoptions /"

			rc=$?

			# A return of 4 or higher means there were serious problems.
			if [ $rc -gt 3 ]; then
				[ -e /proc/splash ] && echo "verbose" > /proc/splash
				# don't use '\n' in nls macro !
				echo
				echo
				nls "*** An error occurred during the file system check."
				nls "*** Dropping you to a shell; the system will reboot"
				nls "*** when you leave the shell."
				echo

				PS1="$(nls '(Repair filesystem)# ')"; export PS1
				[ "$SELINUX" = "1" ] && disable_selinux
				if is_yes "$RUN_SULOGIN_ON_ERR"; then
					/sbin/sulogin
				else
					/bin/sh
				fi

				run_cmd "Unmounting file systems" umount -a
				mount -n -o remount,ro /
				run_cmd "Automatic reboot in progress" reboot
			# A return of 2 or 3 means that filesystem was repaired but we need
			# to reboot.
			elif [ "$rc" = "2" -o "$rc" = "3" ]; then
					[ -e /proc/splash ] && echo "verbose" > /proc/splash
				echo
				nls "*** Filesystem was repaired but system needs to be"
				nls "*** rebooted before mounting it."
				nls "*** REBOOTING ***"
				echo

				run_cmd "Unmounting file systems" umount -a
				mount -n -o remount,ro /
				run_cmd "Automatic reboot in progress" reboot
			elif [ "$rc" = "1" ]; then
				_RUN_QUOTACHECK=1
			fi
		fi

		# Unmount the initrd, if necessary
		if LC_ALL=C grep -q /initrd /proc/mounts && ! LC_ALL=C grep -q /initrd/loopfs /proc/mounts ; then
			if [ -e /initrd/dev/.devfsd ]; then
			umount /initrd/dev
			fi
			umount /initrd
			/sbin/blockdev --flushbufs /dev/ram0 >/dev/null 2>&1
		fi
					  
		# Check for arguments 
		if grep -iq nopnp /proc/cmdline; then
			PNP=
		else
			PNP=yes
		fi

		# set up pnp and kernel pnp
		if [ -n "$PNP" ]; then
			if is_yes "$RUN_USERPNP" && [ -x /sbin/isapnp -a -f /etc/isapnp/isapnp.conf ]; then
			run_cmd "Setting up ISA PNP devices (userspace pnp)" /sbin/isapnp /etc/isapnp/isapnp.conf
			fi
			if is_yes "$RUN_KERNELPNP"; then
			/sbin/modprobe -k isa-pnp 2> /dev/null
			if [ -e /proc/isapnp -a -f /etc/isapnp/isapnp-kernel.conf ]; then
				show "Setting up ISA PNP devices (kernelspace pnp)"; busy
				grep -v "^#" /etc/isapnp/isapnp-kernel.conf > /proc/isapnp && (deltext; ok) || (deltext; fail)
			fi
			fi
		fi

		# Remount the root filesystem read-write
		run_cmd "Remounting root filesystem in rw mode" mount -n -o remount,rw /

		# Update quotas if fsck was run on /
		if [ "$_RUN_QUOTACHECK" = "1" -a -x /sbin/quotacheck ]; then
			run_cmd "Checking root filesystem quotas" /sbin/quotacheck -vnug /
		fi

		# Clean up SELinux labels
		if [ -n "$SELINUX" ]; then
			for file in /etc/mtab /etc/cryptomtab /etc/ld.so.cache ; do
			[ -r $file ] && restorecon $file  >/dev/null 2>&1
			done
		fi
				  
		# Clear mtab
		:>/etc/mtab
		[ -f /etc/cryptomtab ] && :>/etc/cryptomtab

		# Remove stale backups
		rm -f /etc/mtab~ /etc/mtab~~ /etc/cryptomtab~ /etc/cryptomtab~~

		# Enter root, /proc, /sys, devfs and other into mtab.
		mount -f /
		mount -f /proc
		[ -f /proc/bus/usb/devices ] && mount -f -t usbfs usbfs /proc/bus/usb
		[ -e /dev/.devfsd ] && mount -f -t devfs devfs /dev

		if grep -q sysfs /proc/filesystems ; then
			mount -f -t sysfs sysfs /sys
		fi

		if grep -q selinuxfs /proc/filesystems ; then
				mount -f -t selinuxfs selinuxfs /selinux
		fi

# FIXME: should be elsewhere
		# Remove /etc/nologin when starting system
		[ -f /etc/nologin.boot ] && rm -f /etc/nologin /etc/nologin.boot

# $DELAY_LOGIN not handled. defeats the purpose of initng
#		if is_yes "$DELAY_LOGIN" && [ ! -f /etc/nologin ]; then
#			show "Enabling Delay Login"; busy
#			echo > /etc/nologin
#			nls "System bootup in progress - please wait" >> /etc/nologin
#			echo >> /etc/nologin
#			chmod 644 /etc/nologin
#			cp -fp /etc/nologin /etc/nologin.boot
#			ok
#		fi

		exit 0
	}
}
